/*
 * @(#ComboBoxBeanInfo.java
 *
 * Copyright (c) 1997 Symantec Corporation. All Rights Reserved.
 *
 */

package symantec.itools.db.awt;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;


/**
 * BeanInfo for the ComboBox component.
 *
 */

public class ComboBoxBeanInfo extends SimpleBeanInfo {


	/**
	 * Constructs a ComboBoxBeanInfo object.
	 */
	public ComboBoxBeanInfo() {
	}

	/**
	 * Gets BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */
	public BeanInfo[] getAdditionalBeanInfo() {
		try {
			java.util.Vector v = new java.util.Vector();
			BeanInfo[] rv;
			BeanInfo b;
			Class c = beanClass.getSuperclass();

			while (c.isAssignableFrom(Object.class) != true) {
				b = Introspector.getBeanInfo(c);
				v.addElement(b);
				c = c.getSuperclass();
			}
			rv = new BeanInfo[v.size()];
			v.copyInto(rv);

			return rv;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setCanAddChild(false);
		bd.setWinHelp("0x1238E");
		bd.setFolder("dbAWARE");
		bd.setToolbar("dbAWARE");

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("ComboBoxC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("ComboBoxC32.gif");
			return img;
		}

		return null;
	}

    /**
     * Gets an empty MethodDescriptor array.
     * @return an empty array
     */
	public MethodDescriptor[] getMethodDescriptors() {
		java.util.Vector md = new java.util.Vector();
		MethodDescriptor[] rv = new MethodDescriptor[md.size()];
		return rv;
	}

	/**
     * Gets an array of descriptions of this bean's properties.
     * @return property descriptions for this bean
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.db.resources.PropBundle");

		try{

        PropertyDescriptor emptyData = new PropertyDescriptor("emptyMeansNull", beanClass);
		emptyData.setBound(false);
		emptyData.setConstrained(false);
		emptyData.setDisplayName(prop.getString("emptyMeansNull"));

        PropertyDescriptor triggeringEvent = new PropertyDescriptor("triggeringEvent", beanClass);
		triggeringEvent.setBound(false);
		triggeringEvent.setConstrained(false);
		triggeringEvent.setValue("ENUMERATION", "Action_Event=0, Focus_Event=1, Key_Event=2");
		triggeringEvent.setDisplayName(prop.getString("triggeringEvent"));

        PropertyDescriptor dataBinding = new PropertyDescriptor("dataBinding", beanClass);
		dataBinding.setBound(false);
		dataBinding.setConstrained(false);
		dataBinding.setDisplayName(prop.getString("dataBinding"));
		dataBinding.setPropertyEditorClass(symantec.itools.db.beans.binding.NameEditor.class);

        PropertyDescriptor lookupName = new PropertyDescriptor("lookupName", beanClass);
		lookupName.setBound(false);
		lookupName.setConstrained(false);
		lookupName.setDisplayName(prop.getString("lookupName"));
		lookupName.setPropertyEditorClass(symantec.itools.db.beans.binding.NameEditor.class);

		PropertyDescriptor autoExpand = new PropertyDescriptor("autoExpand", beanClass);
		autoExpand.setBound(false);
		autoExpand.setConstrained(false);
		autoExpand.setDisplayName(prop.getString("autoExpand"));

		PropertyDescriptor layout = new PropertyDescriptor("layout", beanClass);
		layout.setHidden(true);

		PropertyDescriptor[] rv = {
		    emptyData,
		    triggeringEvent,
		    lookupName,
		    dataBinding,
		    autoExpand,
		    layout};

		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	/**
	 * Returns the index of the property expected to be changed most often by the designer.
	 */
	public int getDefaultPropertyIndex() {
		return 0;	//  the index for our default property is always 0
	}

	private final static Class beanClass = ComboBox.class;

	}	//  end of class ComboBoxBeanInfo|
