/*
 * @(#ProjectionBean.java
 *
 * Copyright (c) 1997 Symantec Corporation. All Rights Reserved.
 *
 */

package symantec.itools.db.awt;

/**
 * An interface to get/set the database value in a component.
 * dbAWARE components implement this interface.
 */
public interface ProjectionBean {
    /**
     * Sets the value of this component to the given value.
     * @param value the new component value
     * @see #getData
     */
    void setData(Object value);
    /**
     * Gets the value of this component.
     * @return the current component value
     * @see #setData
     */
    Object getData();
    /**
     * Sets whether the data value of this component may be modified.
     * @param value <code>true</code> if the value may not be modified,
     * <code>false</code>if the value may be modified
     */
    void setReadOnly(boolean value);
    /**
     * Gets whether this component saves its value as a text String.
     * @return <code>true</code> if the value is saved as text,
     * <code>false</code> otherwise
     */
    boolean isTextBased();
    /**
     * Gets the number of digits to the right of the decimal point for
     * this component's value.
     * @return the number of digits to the right of the decimal point
     */
    int getScale();
    /**
     * Registers the standard event listener(s) for this component.
     */
    void registerListeners();
}
