/*
 * @(#RadioButton.java
 *
 * Copyright (c) 1997 Symantec Corporation. All Rights Reserved.
 *
 */


package symantec.itools.db.awt;

/**
 * A dbAWARE Checkbox component.
 * <p>
 * This component is similar to the basic Checkbox component, but can
 * be "bound" to a projection within a relation view
 * so that it automatically gets and sets the values in that relation.
 *
 * @see java.awt.CheckboxGroup
 */
public class RadioButton extends java.awt.Checkbox
{

    /**
     * Constructs a default RadioButton.
     * By default the label will read: "<dbAWARE RadioButton>".
     */
    public RadioButton()
    {
        super.setLabel("<dbAWARE RadioButton>");
    }

    /**
     * Constructs a RadioButton with the given label that belongs to the specified
     * RadioBox and has a specific initial value.
     * This call automatically binds this RadioButton to the same RelationView and
     * projection that the RadioBox is bound with.
     *
     * @param label the label for this component
     * @param group the RadioBox this component belongs to
     * @param state the initial value of this component
     */
    public RadioButton(String label, java.awt.CheckboxGroup group, boolean state)
    {
        super(label, group, state);
        if (group instanceof RadioBox) {
            ((RadioBox)group).addCheckbox(this);
        }
    }
}
