/*
 * @(#)SynchronizerLink.java
 *
 * Copyright (c) 1997 Symantec Corporation. All Rights Reserved.
 *
 */

// Each QuerySynchronizer object contains one and only one
// QueryNavigatorTree object.  This tree class implements the JFC
// swing tree interface.  The parent child relationships between tree
// nodes correspond to the master detail relationships between
// QueryNavigators objects.  Each node contains a reference to a
// QueryNavigatorLink object.  The purpose of this link is to hold
// a spot for a QueryNavigator when the alias is known from a detail
// QueryNavigator object before the master QueryNavigator object is
// registered.

package symantec.itools.db.beans.binding;

import javax.awt.swing.tree.*;
import java.beans.PropertyChangeListener;

interface SynchronizerLink
{
    /*
     * Each link holds the alias of it synchronizable object
     */
    String getAlias();
    void setAlias(String value);

    /*
     * Each link holds the master alias of it synchronizable object
     */
    String getMasterAlias();
    void setMasterAlias(String value);

    /*
     * Each link holds the node where it fits into some tree
     */
    TreeNode getNode();
    void setNode(TreeNode value);

    /*
     * Each link holds a synchronizable object
     */
    Synchronizable getSynchronizable();
    void setSynchronizable(Synchronizable value);

    /*
     * Each link holds the synchronizer that owns its tree
     */
    QuerySynchronizer getSynchronizer();
    void setSynchronizer(QuerySynchronizer value);

    /*
     * Invoked by the synchronizable object to perform saveAll
     */
    void saveAll() throws RelationshipPendingException, ParentInvalidRecordException;

    /*
     * Invoked by the synchronizable object to perform saveAllLevels
     */
    void saveAllLevels() throws RelationshipPendingException, ParentInvalidRecordException;

    /*
     * Invoked by the synchronizable object to perform save
     */
    void save() throws RelationshipPendingException, ParentInvalidRecordException;

    /*
     * Invoked by the synchronizable object to perform
     * goto,restart,insert,startqbe,first,next,etc...
     */
    void navigate(int type, Integer position)
        throws ParentInvalidRecordException, RelationshipPendingException;

    /*
     * Is the node saving open space because the synchronizable object isn't added yet
     */
    boolean isOpen();

    /*
     * Remove object references so they can be garbage collected
     */
    void close();

    /*
     * Get link of parent node.
     */
    SynchronizerLink getParentLink();

    /**
     * javadoc comments go here...
     */
    public void addPropertyChangeListener(PropertyChangeListener listener);

    /**
     * javadoc comments go here...
     */
    public void removePropertyChangeListener(PropertyChangeListener listener);

}