/*
 * @(#TextFieldBeanInfo.java
 *
 * Copyright (c) 1997 Symantec Corporation. All Rights Reserved.
 *
 */

package symantec.itools.db.awt;

import java.beans.*;
import symantec.itools.beans.*;
import java.util.ResourceBundle;


/**
 * BeanInfo for the TextField component.
 *
 */

public class TextFieldBeanInfo extends SimpleBeanInfo {

	/**
	 * Constructs a TextFieldBeanInfo object.
	 */
	public TextFieldBeanInfo() {
	}

	/**
	 * Gets BeanInfo for the superclass of this bean.
	 * @return BeanInfo[] containing this bean's superclass BeanInfo
	 */

	public BeanInfo[] getAdditionalBeanInfo() {
		try {
			java.util.Vector v = new java.util.Vector();
			BeanInfo[] rv;
			BeanInfo b;
			Class c = beanClass.getSuperclass();

			while (c.isAssignableFrom(Object.class) != true) {
				b = Introspector.getBeanInfo(c);
				v.addElement(b);
				c = c.getSuperclass();
			}
			rv = new BeanInfo[v.size()];
			v.copyInto(rv);

			return rv;
		}
		catch (IntrospectionException e) { throw new Error(e.toString());}
	}

	/**
	 * Gets the SymantecBeanDescriptor for this bean.
	 * @return an object of type SymantecBeanDescriptor
	 * @see symantec.itools.beans.SymantecBeanDescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		SymantecBeanDescriptor bd = new SymantecBeanDescriptor(beanClass);
		bd.setFolder("dbAWARE");
		bd.setToolbar("dbAWARE");
		// bd.setValue("VPO_Object", new Boolean(true));
		bd.setWinHelp("0x12338");

		return (BeanDescriptor) bd;
	}

	/**
	 * Gets an image that may be used to visually represent this bean
	 * (in the toolbar, on a form, etc).
	 * @param iconKind the type of icon desired, one of: BeanInfo.ICON_MONO_16x16,
	 * BeanInfo.ICON_COLOR_16x16, BeanInfo.ICON_MONO_32x32, or BeanInfo.ICON_COLOR_32x32.
	 * @return an image for this bean, always color even if requested monochrome
	 * @see BeanInfo#ICON_MONO_16x16
	 * @see BeanInfo#ICON_COLOR_16x16
	 * @see BeanInfo#ICON_MONO_32x32
	 * @see BeanInfo#ICON_COLOR_32x32
	 */
	public java.awt.Image getIcon(int iconKind) {
		if (iconKind == BeanInfo.ICON_MONO_16x16 ||
			iconKind == BeanInfo.ICON_COLOR_16x16) {
			java.awt.Image img = loadImage("TextFieldC16.gif");
			return img;
		}

		if (iconKind == BeanInfo.ICON_MONO_32x32 ||
			iconKind == BeanInfo.ICON_COLOR_32x32) {
			java.awt.Image img = loadImage("TextFieldC32.gif");
			return img;
		}

		return null;
	}

    /**
     * Gets an empty MethodDescriptor array.
     * @return an empty array
     */
	public MethodDescriptor[] getMethodDescriptors() {
	    return new MethodDescriptor[0];
	}

	/**
     * Gets an array of descriptions of this bean's properties.
     * @return property descriptions for this bean
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		ResourceBundle prop = ResourceBundle.getBundle("symantec.itools.db.resources.PropBundle");

		try{

        PropertyDescriptor pd1 = new PropertyDescriptor("dataBinding", beanClass);
		pd1.setBound(false);
		pd1.setConstrained(false);
		pd1.setDisplayName(prop.getString("dataBinding"));
		pd1.setPropertyEditorClass(symantec.itools.db.beans.binding.NameEditor.class);

        PropertyDescriptor pd2 = new PropertyDescriptor("triggeringEvent", beanClass);
		pd2.setBound(false);
		pd2.setConstrained(false);
		pd2.setDisplayName(prop.getString("triggeringEvent"));
		pd2.setValue("ENUMERATION", "Focus_Event=1, Text_Event=4, Key_Event=2, Action_Event=0");

        PropertyDescriptor emptyData = new PropertyDescriptor("emptyMeansNull", beanClass);
		emptyData.setBound(false);
		emptyData.setConstrained(false);
		emptyData.setDisplayName(prop.getString("emptyMeansNull"));

		PropertyDescriptor[] rv = {pd1, pd2, emptyData};

		return rv;
		} catch (IntrospectionException e) { throw new Error(e.toString()); }
	}

	/**
	 * Returns the index of the property expected to be changed most often by the designer.
	 */
	public int getDefaultPropertyIndex() {
		return 0;	//  the index for our default property is always 0
	}

	private final static Class beanClass = TextField.class;

}	//  end of class TextFieldBeanInfo|
