/*
 * Copyright (c) 1996 Krumel & Associates, Inc. All Rights Reserved.
 *
 * www.krumel.com - controls@krumel.com
 *
 * Permission is given to the buyer of this package for one software
 * developer to use this software on one CPU (one workstation) and to make
 * one backup copy.  You may uitilize and/or modify this class for use in your
 * projects.  You may distribute or sell any executable which results from
 * using this code in yur application, except a utility or class of similar
 * nature to this product.  You may distribute this product in compiled
 * form only, but soley to be used with your cmpiled executable product
 * for the puposes of dynamic loading. You may NOT redistribute the source
 * code in any form or make it accessible through a network or other
 * distribution media to others. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * The source code is the confidential and proprietary information
 * of Krumel & Associates, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Krumel & Associates, Inc..

 * KRUMEL & ASSOCIATES MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
 * NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. KRUMEL & ASSOCIATES SHALL NOT
 * BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package symantec.itools.db.awt;

import java.awt.Event;
import symantec.itools.db.awt.event.CellEvent;
import symantec.itools.db.awt.event.TableEvent;

/**
 * This interface is implemented by all classes that will serve as event
 * handlers for a TableView. An event handler is an object that receives
 * notification of all events and any exceptions that occur during user
 * interaction with the table view. Events are divided into categories
 * based on what generated the event: cell, row heading, column heading,
 * or toolbar control. Exceptions prevent a unique design challenge for
 * controls.  The table view is designed to run without forcing the
 * developer to register and intercept events; instead,
 */
public interface TvEventHandler extends java.io.Serializable {
    /**
     * Called by the TableView when an event handler is installed.  This
     * method provides the event handler a chance to perform any view
     * initialization and set an instance variable for later use.
     */
    void setupView(TableView v);
    /**
     * Called by the TableView when the user triggers an event that occurs
     * within the context of a normal cell, i.e. not a heading cell.
     * @param e The event object. The source is set to the TableView.
     * @param cell The cell that generated the event.
     */
    void handleCellEvent(CellEvent e, TableCell cell);
    /**
     * Called by the TableView when the user triggers an event that occurs
     * within the context of a column heading.
     * @param e The event object. The source is set to the TableView.
     * @param cell The cell that generated the event.
     */
    void handleColHeadingEvent(CellEvent e, TableCell cell);
    /**
     * Called by the TableView when the user triggers an event that occurs
     * within the context of a row heading heading.
     * @param e The event object. The source is set to the TableView.
     * @param cell The cell that generated the event.
     */
    void handleRowHeadingEvent(CellEvent e, TableCell cell);
    /**
     * Called by the TableView when the user triggers an event that occurs
     * within the context of the corner cell.
     * @param e The event object. The source is set to the TableView.
     * @param cell The cell that generated the event.
     */
    void handleCornerCellEvent(CellEvent e, TableCell cell);
   /**
    * Called by the TableView when the users triggers an event outside
    * the scope of any cell, usually a component located on the toolbar.
    * @param e The unchanged event that was generated.
    */
    void handleTableEvent(TableEvent e);
    /**
     * Called by the TableView when an exception is generated within
     * the TableView framework.  If the exception occured due to some function
     * or event related to a cell, the coordinates of the cell are passed into
     * the method.  This method provides a convinient place to display messages
     * to the user.
     */
    void handleException(Coordinate pos, Exception exc);
}